/******************************************
	PURPOSE:	Tutorial: 4-digit 7-segment LED displays 
	Created by      Rudy Schlaf
	DATE:		1/2014
*******************************************/

#define ontime 3000// the LEDs are on for 3000us per strobe cycle
// Arduino digital pins used to address
// corresponding segments on the LED display
#define A 2
#define B 3
#define C 4
#define D 5
#define E 6
#define f 7
#define G 8
#define DP 9//not used in the circuit shown in the tutorial

// Pins driving common cathodes
#define CC1 10
#define CC2 11
#define CC3 12
#define CC4 13

//definition of segments; digital point is off
//    -------  
//   |   A   |  
//  F|       |B 
//   |---G---|  
//  E|       |C 
//   |   D   |   
//    -------

//each group of 8 in {....} defines the state of the a,b,c,d,e,f,g,dp segments for the numbers from 0 to 9
#define numbersegments { \
  {1,1,1,1,1,1,0,0},\
  {0,1,1,0,0,0,0,0},\
  {1,1,0,1,1,0,1,0},\
  {1,1,1,1,0,0,1,0},\
  {0,1,1,0,0,1,1,0},\
  {1,0,1,1,0,1,1,0},\
  {0,0,1,1,1,1,1,0},\
  {1,1,1,0,0,0,0,0},\
  {1,1,1,1,1,1,1,0},\
  {1,1,1,0,0,1,1,0}\
}

//number array
byte numbers[10][8] = numbersegments;//an array is defined and filled with the 'numbersegments' array defined above. Rows and columns are assigned as follwos: numbers[number][segments]

const int segments[8] = { A, B, C, D, E, f, G, DP };//this defines an array that contains the arduino pins for each segment

void setup() {
  Serial.begin(9600);
  pinMode(A, OUTPUT);//set anodes low to make sure all segments are off
  digitalWrite(A,LOW);
  pinMode(B, OUTPUT);
  digitalWrite(B,LOW);
  pinMode(C, OUTPUT);
  digitalWrite(C,LOW);
  pinMode(D, OUTPUT);
  digitalWrite(D,LOW);
  pinMode(E, OUTPUT);
  digitalWrite(E,LOW);
  pinMode(f, OUTPUT);
  digitalWrite(f,LOW);
  pinMode(G, OUTPUT);
  digitalWrite(G,LOW);
  pinMode(DP, OUTPUT);
  digitalWrite(DP,LOW);

  pinMode(CC1, OUTPUT);//set cathodes high to turn the digits off
  digitalWrite(CC1,HIGH);
  pinMode(CC2, OUTPUT);
  digitalWrite(CC2,HIGH);
  pinMode(CC3, OUTPUT);
  digitalWrite(CC3,HIGH);
  pinMode(CC4, OUTPUT);
  digitalWrite(CC4,HIGH);
  
}

void loop() {
  for (int digit1=0; digit1 < 10; digit1++) {
    for (int digit2=0; digit2 < 10; digit2++) {
      for (int digit3=0; digit3 < 10; digit3++){
        for (int digit4=0; digit4 < 10; digit4++){
          for(int t=0; t<20;t++){//this loop controls the speed of the counter. It repeats the strobe sequence 20 times before the counter counts up again
            //this flashes each segment in sequence. This is the strobing/multiplexing of the digits to be able to show them all in the same time
            //persistence of vision gives the illusion that they are all on at the same time.
            setsegments(digit1,CC1,ontime);//flash digit 1 for 100 microseconds
            setsegments(digit2,CC2,ontime);//flash digit 2 for 100 microseconds...
            setsegments(digit3,CC3,ontime);//...
            setsegments(digit4,CC4,ontime);//...       
          }
        }
      }
    }
  }
}


